/*
 * spi_Comm_Handler.c
 *
 *  Created on: Jun 7, 2013
 *      Author: jgnoss
 */

#include <string.h>

#include "key_debounce.h"
#include "spi_Comm_Handler.h"
#include "f0_u1_driver.h"
#include "printf.h"
#include "stm32_Display.h"

/* Private variables ---------------------------------------------------------*/

volatile uint8_t spi_NextByteToSend = 0;
volatile uint8_t spi_ByteToSendCount = 0;
volatile uint8_t spi_ByteToReceiceCount = 0;

volatile uint8_t spi_DBG_Array[SPI_TRANSFERDATASIZE] = {0, };

volatile spi_TransmissioData_t receivedData;

volatile spi_status_t spiStatus = {
		0, 				// rx_Idx;
		0, 				// tx_Idx;
};


volatile uint8_t spi_RxBuffer[SPI_TRANSFERDATASIZE] = { 0, };
volatile uint8_t spi_TxBuffer[SPI_TRANSFERDATASIZE] = { /*0, };*/

		'0', '1', '2', '3', '4', '5',
		'6', '7', '8', '9', 'A', 'B',
		'C', 'D', 'E', 'F', 'G', 'H',
		'I', 'J', 'K', 'L', 'M', 'N',
		'O', 'P', 'Q', 'R', 'S', 'T',
		'U', 'V', 'W', 'X', 'Y', 'Z' };
/*
spi_TransmissioData_t spi_TransmissioData = {
		49, // cmdByte1;
		{ 50, 51, 52, 53 }, // transTime;
		{ 54, 55, 56, 57 }, // transDate;
		58, // keyByte;
		59, // lcdCmdByte;
		60, // lcdPosX;
		61, // lcdPosY;
		"Text For Pos: 0 0" // lcdLineTxt[21];
		};
*/
spi_TransmissioData_t spi_TransmissioData = {
		sc_NODATA, // cmdByte1;
		{ 19, 34, 12, 1 }, // transTime;
		{ 6, 12, 07, 13 }, // transDate;
		25, // keyByte;
		26, // lcdCmdByte;
		0, // lcdPosX;
		1, // lcdPosY;
		"Text For Pos: 0 0\0", // lcdLineTxt[21];
		0,
		0
		};


SPI_InitTypeDef 		SPI_InitStructure;
volatile spi_status_t	spiStatus;


/**
 * @brief  This function handles SPI interrupt request.
 * @param  None
 * @retval None
 */
void SPI2_IRQHandler(void) {

	/* always check for Bytes to send and send it */
	if (SPI_I2S_GetITStatus(SPIx, SPI_I2S_IT_TXE ) == SET) {
		if (spiStatus.tx_Idx < SPI_TRANSFERDATASIZE) {
			spi_DBG_Array[spiStatus.tx_Idx] = ((uint8_t *)&spi_TransmissioData)[spiStatus.tx_Idx];
			SPI_SendData8(SPIx, ((uint8_t *)&spi_TransmissioData)[spiStatus.tx_Idx++]);
		} else {
			/* nothing to send? disable TXE IRQ */
			SPI_I2S_ITConfig(SPIx, SPI_I2S_IT_TXE, DISABLE);

		}
	}


	/* always get received Byte to reset the RXNE Flag */
	if (SPI_I2S_GetITStatus(SPIx, SPI_I2S_IT_RXNE ) == SET) {
		if (spiStatus.rx_Idx < SPI_TRANSFERDATASIZE) {
			spi_RxBuffer[spiStatus.rx_Idx++] = SPI_ReceiveData8(SPIx );
		}
	}


	/* SPI Error interrupt--------------------------------------- */
	if (SPI_I2S_GetITStatus(SPIx, SPI_I2S_IT_OVR ) == SET) {
		// OVR Flag gets reset by
		// reading SPI_DR
		SPI_ReceiveData8(SPIx );
		// followed by reading SPI_SR
		SPI_I2S_GetITStatus(SPIx, SPI_I2S_IT_OVR );
	}
}

void init_SPI(void) {

//	uart1_puts("init_SPI \r\n> ");

	/* SPI configuration ------------------------------------------------------*/
	config_SPI_IO();

	/* Initializes the SPI communication */
	SPI_InitStructure.SPI_Mode = SPI_Mode_Slave;
	SPI_Init(SPIx, &SPI_InitStructure);

	/* Initialize the FIFO threshold */
	SPI_RxFIFOThresholdConfig(SPIx, SPI_RxFIFOThreshold_QF );

	/* Enable the Rx buffer not empty interrupt */
	SPI_I2S_ITConfig(SPIx, SPI_I2S_IT_RXNE, ENABLE);
	/* Enable the SPI Error interrupt */
	SPI_I2S_ITConfig(SPIx, SPI_I2S_IT_ERR, ENABLE);
	/* Data transfer is performed in the SPI interrupt routine */

	/* Enable the SPI peripheral */
	SPI_Cmd(SPIx, ENABLE);

	flush_SPI_RX_Buffer();

	spiStatus.rx_Idx = 0;
	spiStatus.tx_Idx = 0;


	SPI_SendData8(SPIx, spi_TxBuffer[spiStatus.tx_Idx++]);
	SPI_I2S_ITConfig(SPIx, SPI_I2S_IT_TXE, ENABLE);

}

/**
 * @brief  Configures the SPI Peripheral.
 * @param  None
 * @retval None
 */
void config_SPI_IO(void) {
	GPIO_InitTypeDef GPIO_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	/* Enable the SPI periph */
	if (SPIx == SPI1 ) {
		RCC_APB2PeriphClockCmd(SPIx_CLK, ENABLE);
	} else {
		RCC_APB1PeriphClockCmd(SPIx_CLK, ENABLE);
	}

	/* Enable SCK, MOSI, MISO and NSS GPIO clocks */
	RCC_AHBPeriphClockCmd(
			SPIx_SS_GPIO_CLK | SPIx_SCK_GPIO_CLK | SPIx_MISO_GPIO_CLK
					| SPIx_MOSI_GPIO_CLK, ENABLE);

	GPIO_PinAFConfig(SPIx_SS_GPIO_PORT, SPIx_SS_SOURCE, SPIx_SS_AF );
	GPIO_PinAFConfig(SPIx_SCK_GPIO_PORT, SPIx_SCK_SOURCE, SPIx_SCK_AF );
	GPIO_PinAFConfig(SPIx_MOSI_GPIO_PORT, SPIx_MOSI_SOURCE, SPIx_MOSI_AF );
	GPIO_PinAFConfig(SPIx_MISO_GPIO_PORT, SPIx_MISO_SOURCE, SPIx_MISO_AF );

	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_Level_3;

	/* SPI SS pin configuration */
	GPIO_InitStructure.GPIO_Pin = SPIx_SS_PIN;
	GPIO_Init(SPIx_SS_GPIO_PORT, &GPIO_InitStructure);

	/* SPI SCK pin configuration */
	GPIO_InitStructure.GPIO_Pin = SPIx_SCK_PIN;
	GPIO_Init(SPIx_SCK_GPIO_PORT, &GPIO_InitStructure);

	/* SPI  MOSI pin configuration */
	GPIO_InitStructure.GPIO_Pin = SPIx_MOSI_PIN;
	GPIO_Init(SPIx_MOSI_GPIO_PORT, &GPIO_InitStructure);

	/* SPI MISO pin configuration */
	GPIO_InitStructure.GPIO_Pin = SPIx_MISO_PIN;
	GPIO_Init(SPIx_MISO_GPIO_PORT, &GPIO_InitStructure);

	/* SPI configuration -------------------------------------------------------*/
	SPI_I2S_DeInit(SPIx );
	SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
	SPI_InitStructure.SPI_DataSize = SPI_DATASIZE;
	SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
	SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
	SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;		// SPI_NSS_Hard;
	SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;
	SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
	SPI_InitStructure.SPI_CRCPolynomial = 7;

	/* Configure the SPI interrupt priority */
	NVIC_InitStructure.NVIC_IRQChannel = SPIx_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);


	/* configure the Master Knock Line */
	RCC_AHBPeriphClockCmd(SPIx_KNOCK_GPIO_CLK, ENABLE);

	GPIO_InitStructure.GPIO_Pin = SPIx_KNOCK_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &GPIO_InitStructure);


/*
	//NSS Config
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
*/
	/* SPI SCK pin configuration */
//	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
//	GPIO_Init(GPIOB, &GPIO_InitStructure);

}


uint8_t spi_MasterSendData(uint8_t *data, uint8_t bytes) {
	char ss[32] = "";

	/* Check if TX FIFO is empty */
	if (SPI_GetTransmissionFIFOStatus(SPIx ) != SPI_TransmissionFIFOStatus_Empty ) {
		uart1_puts("TX FIFO not empty\r\n> ");
		return 1;
	}

	/* check busy flag */
	if (SPI_I2S_GetFlagStatus(SPIx, SPI_I2S_FLAG_BSY ) == SET) {
		uart1_puts("SPI busy \r\n> ");
		return 2;
	}

	/* check for RX FIFO is empty */
	if (SPI_GetReceptionFIFOStatus(SPIx ) != SPI_ReceptionFIFOStatus_Empty ) {
		uart1_puts("RX FIFO not empty\r\n> ");
		return 3;
	}

	flush_SPI_RX_Buffer();

	spiStatus.rx_Idx = 0;
	spiStatus.tx_Idx = 0;


	SPI_SendData8(SPIx, spi_TxBuffer[spiStatus.tx_Idx++]);
	SPI_I2S_ITConfig(SPIx, SPI_I2S_IT_TXE, ENABLE);
	return 0;
//FIXME
}

void knockMaster(void){
	uint8_t c = 10;
	MASTER_KNOCK_ON();
	while (c) {
		c--;
	}
	MASTER_KNOCK_OFF();
}

void spi_SlaveSendDateTimeData(void){

	RTC_GetDate(RTC_Format_BIN, &spi_TransmissioData.transDate);
	RTC_GetTime(RTC_Format_BIN, &spi_TransmissioData.transTime);
	spi_TransmissioData.cmdByte1 = sc_SlaveDateTime;
	spi_TransmissioData.keyByte = key_state;
	spi_SlaveSendData();
}

void spi_SlaveSendKeyData(uint8_t kd){
	char dd[20] = "";
//	flush_SPI_TX_Buffer();
	memset(&spi_TransmissioData, 0, sizeof(spi_TransmissioData));
	spi_TransmissioData.cmdByte1 = sc_KeyByte;
	spi_TransmissioData.keyByte = kd;

	tfp_sprintf(dd, "\r\nKey %d  %d\r\n> ", kd, spi_TransmissioData.keyByte);
	uart1_puts(dd);

	spi_SlaveSendData();
}


void spi_SlaveSendData(void){
	char dd[25] = "";
	int c = 0;

	/* clear Receive Buffer */
	flush_SPI_RX_Buffer();

	spiStatus.tx_Idx = 0;

	for (c = 0; c < sizeof(spi_TransmissioData); c++){
		((uint8_t *)&spi_TransmissioData)[c] = 48 + c;
	}

	tfp_sprintf(dd, "\r\nload %d to send\r\n> ", ((uint8_t *)&spi_TransmissioData)[spiStatus.tx_Idx]);
	uart1_puts(dd);
	spi_DBG_Array[0] = ((uint8_t *)&spi_TransmissioData)[spiStatus.tx_Idx];
	SPI_SendData8(SPIx, ((uint8_t *)&spi_TransmissioData)[spiStatus.tx_Idx++]);
	SPI_I2S_ITConfig(SPIx, SPI_I2S_IT_TXE, ENABLE);
	knockMaster();
}

void clear_Buffer(uint8_t *pBuffer, uint16_t BufferLength) {
	uint16_t index = 0;

	/* Put in global buffer same values */
	for (index = 0; index < BufferLength; index++) {
		pBuffer[index] = 0x00;
	}
}

void flush_SPI_RX_Buffer(void) {
	clear_Buffer((uint8_t *)&spi_RxBuffer, SPI_TRANSFERDATASIZE);
	spiStatus.rx_Idx = 0;
}

void flush_SPI_TX_Buffer(void) {
	clear_Buffer((uint8_t *)spi_TxBuffer, SPI_TRANSFERDATASIZE);
	spiStatus.tx_Idx = 0;
}

/*
 * to prevent recursive traffic
 * here we cannot call to transfer another data
 */
void handle_incoming_SPI_Data(void) {
	char ss[64] = "";
	uart1_putc(receivedData.cmdByte1+48);
	uart1_putc(59);

	switch (receivedData.cmdByte1) {
		case sc_NODATA:
			uart1_puts("got : sc_NODATA\r\n");
		break;
		case sc_ACK:
			uart1_puts("got : sc_ACK\r\n");
		break;
		case sc_NACK:
			uart1_puts("got : sc_NACK\r\n");
		break;
		case sc_WriteToLCD:
			//stm32_WriteToDisplayLine
			stm32_WriteToDisplay_XY(receivedData.lcdPosX, receivedData.lcdPosY,  (char *)receivedData.lcdLineTxt);
			/**/
			break;
		case sc_KeyByte:
			key_state = receivedData.keyByte;
		break;
		case sc_SlaveDateTime:
			uart1_puts("giving time here\r\n");
		break;
		case sc_MasterDateTime:
			tfp_sprintf (ss, "got new Time\n"
					"transTime	: %d:%d:%d \r\n"
					"transDate	: %d.%d.%d \r\n",
					receivedData.transTime.RTC_Hours,
					receivedData.transTime.RTC_Minutes,
					receivedData.transTime.RTC_Seconds,
					receivedData.transDate.RTC_Date,
					receivedData.transDate.RTC_Month,
					receivedData.transDate.RTC_Year);
			uart1_puts(ss);

		break;
	}

}

void showRData(spi_TransmissioData_t tdata) {
	char tt[50] = "";

	tfp_sprintf (tt, "cmdByte1	: %d \r\n", tdata.cmdByte1);
	uart1_puts(tt);
	tfp_sprintf (tt, "transTime	: %d:%d:%d \r\n",
			tdata.transTime.RTC_Hours,
			tdata.transTime.RTC_Minutes,
			tdata.transTime.RTC_Seconds);
	uart1_puts(tt);
	tfp_sprintf (tt, "transDate	: %d.%d.%d \r\n",
			tdata.transDate.RTC_Date,
			tdata.transDate.RTC_Month,
			tdata.transDate.RTC_Year);
	uart1_puts(tt);
	tfp_sprintf (tt, "keyByte	: %d \r\n", tdata.keyByte);
	uart1_puts(tt);
	tfp_sprintf (tt, "lcdCmdByte	: %d \r\n", tdata.lcdCmdByte);
	uart1_puts(tt);
	tfp_sprintf (tt, "lcdPosX	: %d \r\n", tdata.lcdPosX);
	uart1_puts(tt);
	tfp_sprintf (tt, "lcdPosY	: %d \r\n", tdata.lcdPosY);
	uart1_puts(tt);
	tfp_sprintf (tt, "lcdLineTxt	: %s \r\n", tdata.lcdLineTxt);
	uart1_puts(tt);

}


void handledData(spi_TransmissioData_t tdata){
	showRData(tdata);
	switch (tdata.cmdByte1) {
	case sc_WriteToLCD :

		stm32_WriteToDisplay_XY(tdata.lcdPosY, tdata.lcdPosX, (char *)tdata.lcdLineTxt);

		break;
	case sc_KeyByte :

		break;
	}
}


void handleReceivedData(void){

	memcpy((char *)&receivedData, (char *)spi_RxBuffer, SPI_TRANSFERDATASIZE);

	handle_incoming_SPI_Data();

	flush_SPI_RX_Buffer();

	// FIXME we prepare new transmit data here maybe another place is better to do this
	spiStatus.tx_Idx = 0;
	SPI_SendData8(SPIx, ((uint8_t *)&spi_TransmissioData)[spiStatus.tx_Idx++]);
	SPI_I2S_ITConfig(SPIx, SPI_I2S_IT_TXE, ENABLE);
}
