/*
 * spi_Comm_Handler.h
 *
 *  Created on: Jun 7, 2013
 *      Author: jgnoss
 *
 * @brief  How Communication works in that One Master One Slave App
 *
 * Initial State Master:
 * 		configured as Master
 * 		no Data Loaded to RX and TX Buffer
 * 		Activated Interrupt on TXE and RXNE
 *
 * Initial State Slave:
 * 		configured as Slave
 * 		Activated Interrupt RXNE
 * 		sc_ACK Loaded to TX Buffer
 * 		Activated Interrupt TXE
 *
 * 1.	Master start sending a datagramm ,
 * 		1. Byte		;
 * 		2. Byte		Datalength
 * 		xx Bytes	Data
 * 		xx+1		;
 * 		xx+2		0x00
 *
 *
 *
 *
 */


#ifndef SPI_COMM_HANDLER_H_
#define SPI_COMM_HANDLER_H_

#include "stm32f0xx_conf.h"

/*  only one should be activated */
#define SPI_COMM_MASTER
//#define SPI_COMM_SLAVE


#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))

/* Communication boards SPIx Interface */
#define SPIx                             SPI2
#define SPIx_CLK                         RCC_APB1Periph_SPI2
#define SPIx_IRQn                        SPI2_IRQn
#define SPIx_IRQHandler                  SPI2_IRQHandler


#define SPIx_SS_PIN                      GPIO_Pin_12
#define SPIx_SS_GPIO_PORT                GPIOB
#define SPIx_SS_GPIO_CLK                 RCC_AHBPeriph_GPIOB
#define SPIx_SS_SOURCE                   GPIO_PinSource12
#define SPIx_SS_AF                       GPIO_AF_0

#define SPIx_SCK_PIN                     GPIO_Pin_13
#define SPIx_SCK_GPIO_PORT               GPIOB
#define SPIx_SCK_GPIO_CLK                RCC_AHBPeriph_GPIOB
#define SPIx_SCK_SOURCE                  GPIO_PinSource13
#define SPIx_SCK_AF                      GPIO_AF_0

#define SPIx_MISO_PIN                    GPIO_Pin_14
#define SPIx_MISO_GPIO_PORT              GPIOB
#define SPIx_MISO_GPIO_CLK               RCC_AHBPeriph_GPIOB
#define SPIx_MISO_SOURCE                 GPIO_PinSource14
#define SPIx_MISO_AF                     GPIO_AF_0

#define SPIx_MOSI_PIN                    GPIO_Pin_15
#define SPIx_MOSI_GPIO_PORT              GPIOB
#define SPIx_MOSI_GPIO_CLK               RCC_AHBPeriph_GPIOB
#define SPIx_MOSI_SOURCE                 GPIO_PinSource15
#define SPIx_MOSI_AF                     GPIO_AF_0

#define SPIx_KNOCK_PIN                    GPIO_Pin_8
#define SPIx_KNOCK_GPIO_PORT              GPIOA
#define SPIx_KNOCK_GPIO_CLK               RCC_AHBPeriph_GPIOA
#define SPIx_KNOCK_SOURCE                 GPIO_PinSource8
#define SPIx_KNOCK_BSRR					  GPIOA->BSRR
#define SPIx_KNOCK_SET					  GPIO_BSRR_BS_8
#define SPIx_KNOCK_RESET				  GPIO_BSRR_BR_8

#define MASTER_KNOCK_ON()				(SPIx_KNOCK_BSRR = SPIx_KNOCK_SET)
#define MASTER_KNOCK_OFF()				(SPIx_KNOCK_BSRR = SPIx_KNOCK_RESET)


/* Define data size and data masks for 8 Bit */
 #define SPI_DATASIZE                     SPI_DataSize_8b
 #define SPI_DATAMASK                     (uint8_t)0xFF

#define SPI_TRANSFERDATASIZE				36


typedef struct {
	uint8_t				cmdByte1;
	RTC_TimeTypeDef		transTime;
	RTC_DateTypeDef		transDate;
	uint8_t				keyByte;
	uint8_t				lcdCmdByte;
	uint8_t				lcdPosX;
	uint8_t				lcdPosY;
	uint8_t				lcdLineTxt[21];
	uint8_t				dummy1;
	uint8_t				dummy2;
} spi_TransmissioData_t;

typedef enum {
	sc_NODATA,
	sc_ACK,
	sc_NACK,
	sc_WriteToLCD,
	sc_KeyByte,
	sc_SlaveDateTime,
	sc_MasterDateTime
} spi_command_t;

typedef enum {
	spiTSSt_Idle,
	spiTSSt_Prefix,
	spiTSSt_DataCount,
	spiTSSt_Data,
	spiTSSt_postfix
} spi_TransmitStatus_t;

typedef enum {
	spiRSSt_Idle,
	spiRSSt_Prefix,
	spiRSSt_DataCount,
	spiRSSt_Data,
	spiRSSt_postfix
} spi_ReceiveStatus_t;


typedef  struct {
	uint8_t					rx_Idx;
	uint8_t 				tx_Idx;
} spi_status_t;


//extern volatile spi_TransmissioData_t spi_RxBuffer;
extern volatile	uint8_t 		spi_TxBuffer[];
extern volatile	uint8_t 		spi_RxBuffer[];
extern volatile spi_status_t	spiStatus;

//extern volatile spi_TransmissioData_t 		spi_TransmissionData;


void init_SPI(void);
void config_SPI_IO(void);

uint8_t spi_MasterSendData(uint8_t *data, uint8_t bytes);

void spi_SlaveSendDateTimeData(void);
void spi_SlaveSendKeyData(uint8_t kd);
void spi_SlaveSendData(void);



void knockMaster(void);
void clear_Buffer(uint8_t *pBuffer, uint16_t BufferLength);
void flush_SPI_RX_Buffer(void);
void flush_SPI_TX_Buffer(void);

void showRData(spi_TransmissioData_t tdata);
void handledData(spi_TransmissioData_t tdata);
void handleReceivedData(void);

#endif /* SPI_COMM_HANDLER_H_ */
